import argparse
import logging
import sys

from .tool import Kapten


def command(input_args=None):
    if input_args is None:
        input_args = sys.argv[1:]

    parser = argparse.ArgumentParser(
        description="Checks for new images and updates services if needed."
    )
    parser.add_argument(
        "-s",
        "--service",
        type=str,
        action="append",
        dest="services",
        required=True,
        help="Service to update",
    )
    parser.add_argument("--slack", type=str, help="Slack token to use for notification")
    parser.add_argument(
        "--check", action="store_true", help="Only check if service needs to be updated"
    )
    parser.add_argument("--force", action="store_true", help="Force service update")
    parser.add_argument(
        "-v", "--verbosity", type=int, help="Level of verbosity", default=1
    )

    args = parser.parse_args(input_args)

    level = logging.INFO
    if args.verbosity == 0:
        level = logging.CRITICAL
    elif args.verbosity > 1:
        level = logging.DEBUG

    logging.basicConfig(level=level, format="%(asctime)s - %(message)s")

    client = Kapten(
        args.services, slack_token=args.slack, only_check=args.check, force=args.force
    )
    client.update_services()



import logging

from docker.api import APIClient

from . import slack

logger = logging.getLogger(__name__)


class Kapten(object):
    def __init__(self, service_names, slack_token=None, only_check=False, force=False):
        self.service_names = service_names
        self.slack_token = slack_token
        self.only_check = only_check
        self.force = force
        self.client = APIClient()

    def get_latest_digest(self, image_name):
        data = self.client.inspect_distribution(image_name)
        digest = data["Descriptor"]["digest"]
        return digest

    def update_service(self, spec):
        service_id = spec["ID"]
        service_version = spec["Version"]["Index"]
        service_name = spec["Spec"]["Name"]
        task_template = spec["Spec"]["TaskTemplate"]
        container_spec = task_template["ContainerSpec"]
        stack = container_spec["Labels"].get("com.docker.stack.namespace", None)
        service_short_name = (
            service_name[len(stack) + 1 :]
            if service_name.startswith(stack + "_")
            else service_name
        )
        container_image = container_spec["Image"]
        image_name, _, current_digest = container_image.partition("@")

        # Fetch latest image digest
        latest_digest = self.get_latest_digest(image_name)
        latest_image = "{}@{}".format(image_name, latest_digest)

        logger.debug("Stack:     %s", stack or "-")
        logger.debug("Service:   %s", service_short_name)
        logger.debug("Image:     %s", image_name)
        logger.debug("  Current: %s", current_digest)
        logger.debug("  Latest:  %s", latest_digest)

        if self.force or latest_digest != current_digest:
            if self.only_check:
                logger.info("Can update service %s to %s", service_name, latest_image)
                return

            logger.info("Updating service %s to %s", service_name, latest_image)

            # Update service to latest image
            task_template["ContainerSpec"]["Image"] = latest_image
            self.client.update_service(
                service_id,
                service_version,
                task_template=task_template,
                fetch_current_spec=True,
            )

            # Notify slack
            if self.slack_token:
                slack.notify(
                    self.slack_token,
                    service_name,
                    latest_digest,
                    stack=stack,
                    service_short_name=service_short_name,
                    image_name=image_name,
                )

    def update_services(self):
        service_specs = self.client.services({"name": self.service_names})
        for service_spec in service_specs:
            try:
                self.update_service(service_spec)
            except Exception as e:
                logger.critical(
                    "Failed to update service %s: %s",
                    service_spec["Spec"]["Name"],
                    e.message,
                )



#!/usr/bin/env python
import codecs
from os import path

from setuptools import setup

# Get version from package
version = __import__("kapten").__version__

# Get the long description from the README
long_description = None
here = path.dirname(path.abspath(__file__))
with codecs.open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="kapten",
    version=version,
    author="Jonas Lundberg",
    author_email="jonas@5monkeys.se",
    url="https://github.com/5monkeys/kapten",
    license="MIT",
    keywords=["docker", "swarm", "stack", "service", "auto", "deploy"],
    description="Auto deploy of Docker Swarm services",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    packages=["kapten"],
    entry_points={"console_scripts": ["kapten = kapten.cli:command"]},
    install_requires=["docker"],
    tests_require=["coverage", "responses"],
    test_suite="tests",
)



import logging

import requests

logger = logging.getLogger(__name__)


def post(token, text, fields=None, fallback=None):
    logger.debug("Notifying Slack...")
    payload = {"text": text}
    if fields:
        payload["attachments"] = [
            {"color": "#50ba32", "fallback": fallback or text, "fields": fields}
        ]
    response = requests.post(
        "https://hooks.slack.com/services/{}".format(token), json=payload
    )
    return response.text == "ok"


def notify(token, service_name, image_digest, **kwargs):
    fields = []
    if "stack" in kwargs:
        fields.append({"title": "Stack", "value": kwargs["stack"], "short": True})

    fields.append(
        {
            "title": "Service",
            "value": kwargs.get("service_short_name", service_name),
            "short": True,
        }
    )

    if "image_name" in kwargs:
        fields.append({"title": "Image", "value": kwargs["image_name"], "short": False})

    fields.append({"title": "Digest", "value": image_digest, "short": False})

    return post(
        token,
        "Deployment of *{}* has started.".format(service_name),
        fallback="Deploying {}, {}".format(service_name, image_digest),
        fields=fields,
    )
